@@ -1,22 +1,21 @@
-/* gdaui-data-cell-renderer-textual.c
- *
- * Copyright (C) 2000  Red Hat, Inc.,  Jonathan Blandford <jrb@redhat.com>
- * Copyright (C) 2003 - 2010 Vivien Malerba <malerba@gnome-db.org>
+/*
+ * Copyright (C) 2009 - 2010 Vivien Malerba <malerba@gnome-db.org>
+ * Copyright (C) 2010 David King <davidk@openismus.com>
  *
  * This library is free software; you can redistribute it and/or
- * modify it under the terms of the GNU Library General Public
+ * modify it under the terms of the GNU Lesser General Public
  * License as published by the Free Software Foundation; either
  * version 2 of the License, or (at your option) any later version.
  *
  * This library is distributed in the hope that it will be useful,
  * but WITHOUT ANY WARRANTY; without even the implied warranty of
  * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
- * Library General Public License for more details.
+ * Lesser General Public License for more details.
  *
- * You should have received a copy of the GNU Library General Public
+ * You should have received a copy of the GNU Lesser General Public
  * License along with this library; if not, write to the
- * Free Software Foundation, Inc., 59 Temple Place - Suite 330,
- * Boston, MA 02111-1307, USA.
+ * Free Software Foundation, Inc., 51 Franklin St, Fifth Floor,
+ * Boston, MA  02110-1301, USA.
  */
 
 #define OPTIMIZE 0
@@ -460,7 +459,7 @@ gdaui_data_cell_renderer_textual_set_property (GObject *object,
 					str = gda_data_handler_get_str_from_value (datacell->priv->dh, gval);
 					gboolean use_markup = FALSE;
 					if (str) {
-						gint length;
+						guint length;
 						length = g_utf8_strlen (str, -1);
 						if (length > MAX_ACCEPTED_STRING_LENGTH + too_long_msg_len) {
 							gchar *tmp;
@@ -573,7 +572,7 @@ gdaui_data_cell_renderer_textual_set_property (GObject *object,
 
 /**
  * gdaui_data_cell_renderer_textual_new:
- * @dh: a #GdaDataHandler object, or %NULL
+ * @dh: (allow-none): a #GdaDataHandler object, or %NULL
  * @type: the #GType being edited
  * @options: options as a string
  *
@@ -585,7 +584,7 @@ gdaui_data_cell_renderer_textual_set_property (GObject *object,
  * value in the model, thus rendering a different string in each row
  * of the #GtkTreeView
  *
- * Return value: (transfer full): the new cell renderer
+ * Returns: (transfer full): the new cell renderer
  **/
 GtkCellRenderer *
 gdaui_data_cell_renderer_textual_new (GdaDataHandler *dh, GType type, const gchar *options)
@@ -629,20 +628,13 @@ gdaui_data_cell_renderer_textual_render (GtkCellRenderer      *cell,
 	(text_class->render) (cell, window, widget, background_area, cell_area, expose_area, flags);
 
 	if (datacell->priv->to_be_deleted) {
-		GtkStyle *style;
-		guint xpad;
-
-		g_object_get ((GObject*) widget, "style", &style, NULL);
-		g_object_get ((GObject*) cell, "xpad", &xpad, NULL);
-
-		gtk_paint_hline (style,
-				 window, GTK_STATE_SELECTED,
-				 cell_area,
-				 widget,
-				 "hline",
-				 cell_area->x + xpad, cell_area->x + cell_area->width - xpad,
-				 cell_area->y + cell_area->height / 2.);
-		g_object_unref (style);
+		cairo_t *cr;
+		cr = gdk_cairo_create (window);
+		cairo_set_source_rgba (cr, 0., 0., 0., 1.);
+		cairo_rectangle (cr, cell_area->x, cell_area->y + cell_area->height / 2. - .5,
+				 cell_area->width, 1.);
+		cairo_fill (cr);
+		cairo_destroy (cr);
 	}
 	if (datacell->priv->invalid)
 		gdaui_data_cell_renderer_draw_invalid_area (window, cell_area);
