@@ -47,6 +47,7 @@ static void gdaui_basic_form_get_property (GObject *object,
 					   guint param_id,
 					   GValue *value,
 					   GParamSpec *pspec);
+static void gdaui_basic_form_widget_grab_focus (GtkWidget *widget);
 
 typedef struct {
 	GdaHolder *holder;
@@ -168,6 +169,12 @@ struct _GdauiBasicFormPriv
 	GSList     *size_groups; /* list of SizeGroup pointers */
 
 	GtkWidget  *mainbox;
+
+	/* unknown value color */
+	gdouble     red;
+	gdouble     green;
+	gdouble     blue;
+	gdouble     alpha;
 };
 
 
@@ -202,11 +209,12 @@ gdaui_basic_form_get_type (void)
 }
 
 static void
-gdaui_basic_form_class_init (GdauiBasicFormClass * class)
+gdaui_basic_form_class_init (GdauiBasicFormClass *klass)
 {
-	GObjectClass   *object_class = G_OBJECT_CLASS (class);
+	GObjectClass *object_class = G_OBJECT_CLASS (klass);
 
-	parent_class = g_type_class_peek_parent (class);
+	parent_class = g_type_class_peek_parent (klass);
+	GTK_WIDGET_CLASS (klass)->grab_focus = gdaui_basic_form_widget_grab_focus;
 
 	/* signals */
 	/**
@@ -272,9 +280,9 @@ gdaui_basic_form_class_init (GdauiBasicFormClass * class)
                               _gdaui_marshal_VOID__OBJECT, G_TYPE_NONE,
 			      1, GTK_TYPE_MENU);
 
-	class->holder_changed = NULL;
-	class->activated = NULL;
-	class->layout_changed = NULL;
+	klass->holder_changed = NULL;
+	klass->activated = NULL;
+	klass->layout_changed = NULL;
 	object_class->dispose = gdaui_basic_form_dispose;
 
 	/* Properties */
@@ -412,6 +420,11 @@ gdaui_basic_form_init (GdauiBasicForm *wid)
 			  G_CALLBACK (popup_menu_cb), wid);
 	g_signal_connect (evbox, "button-press-event",
 			  G_CALLBACK (button_press_event_cb), wid);
+
+	wid->priv->red = .98;
+	wid->priv->green = .93;
+	wid->priv->blue = .25;
+	wid->priv->alpha = .50;
 }
 
 /**
@@ -1056,6 +1069,9 @@ create_entry_widget (SingleEntry *sentry)
 	sentry->entry = GDAUI_DATA_ENTRY (entry);
 	g_object_ref_sink (sentry->entry);
 	gdaui_data_entry_set_editable (sentry->entry, editable);
+	gdaui_data_entry_set_unknown_color (sentry->entry, sentry->form->priv->red,
+					    sentry->form->priv->green, sentry->form->priv->blue,
+					    sentry->form->priv->alpha);
 
 	GSList *list;
 	for (list = sentry->form->priv->size_groups; list; list = list->next) {
@@ -1116,6 +1132,8 @@ create_entries (GdauiBasicForm *form)
 	gdaui_basic_form_show_entry_actions (form, form->priv->show_actions);
 	/* Set the Auto entries default in the entries */
 	gdaui_basic_form_set_entries_auto_default (form, form->priv->entries_auto_default);
+
+	gdaui_basic_form_reset (form);
 }
 
 static void
@@ -1640,6 +1658,16 @@ parameter_changed_cb (GdaHolder *param, SingleEntry *sentry)
 	else
 		gdaui_entry_shell_set_unknown (GDAUI_ENTRY_SHELL (entry),
 					       !gda_holder_is_valid (param));
+
+	/* correctly update the NULLOK status */
+	GdaValueAttribute attr;
+	gboolean nullok;
+	attr = gdaui_data_entry_get_attributes (entry);
+	nullok = !gda_holder_get_not_null (param);
+	if (((attr & GDA_VALUE_ATTR_CAN_BE_NULL) && !nullok) ||
+	    (! (attr & GDA_VALUE_ATTR_CAN_BE_NULL) && nullok))
+		gdaui_data_entry_set_attributes (entry, nullok ? GDA_VALUE_ATTR_CAN_BE_NULL : 0,
+						 GDA_VALUE_ATTR_CAN_BE_NULL);
 }
 
 /**
@@ -1900,24 +1928,46 @@ gdaui_basic_form_entry_set_visible (GdauiBasicForm *form, GdaHolder *param, gboo
 	sentry->prog_hidden = !show;
 }
 
+static void
+gdaui_basic_form_widget_grab_focus (GtkWidget *widget)
+{
+	gdaui_basic_form_entry_grab_focus (GDAUI_BASIC_FORM (widget), NULL);
+}
+
 /**
  * gdaui_basic_form_entry_grab_focus:
  * @form: a #GdauiBasicForm widget
- * @param: a #GdaHolder object
+ * @param: (allow-none): a #GdaHolder object, or %NULL
  *
- * Makes the data entry corresponding to @param grab the focus for the window it's in
+ * Makes the data entry corresponding to @param grab the focus for the window it's in. If @param is %NULL,
+ * then the focus is on the first entry which needs attention.
  *
  * Since: 4.2
  */
 void
 gdaui_basic_form_entry_grab_focus (GdauiBasicForm *form, GdaHolder *param)
 {
-	GtkWidget *entry;
+	GtkWidget *entry = NULL;
 
 	g_return_if_fail (GDAUI_IS_BASIC_FORM (form));
 
-	entry = gdaui_basic_form_get_entry_widget (form, param);
+	if (param) {
+		g_return_if_fail (GDA_IS_HOLDER (param));
+		entry = gdaui_basic_form_get_entry_widget (form, param);
+	}
 
+	if (!entry && form->priv->set) {
+		GSList *list;
+		for (list = form->priv->set->holders; list; list = list->next) {
+			GdaHolder *holder;
+			holder = GDA_HOLDER (list->data);
+			if (!gda_holder_is_valid (holder)) {
+				entry = gdaui_basic_form_get_entry_widget (form, holder);
+				if (entry)
+					break;
+			}
+		}
+	}
 	if (entry)
 		gdaui_data_entry_grab_focus (GDAUI_DATA_ENTRY (entry));
 }
@@ -2375,3 +2425,36 @@ gdaui_basic_form_remove_from_size_group (GdauiBasicForm *form, GtkSizeGroup *siz
 		}
 	}
 }
+
+/**
+ * gdaui_basic_form_set_unknown_color:
+ * @form: a #GdauiBasicForm widget
+ * @red: the red component of a color
+ * @green: the green component of a color
+ * @blue: the blue component of a color
+ * @alpha: the alpha component of a color
+ *
+ * Defines the color to be used when @form displays an invalid value. Any value not
+ * between 0. and 1. will result in the default hard coded values to be used (grayish).
+ *
+ * Since: 4.2.13
+ */
+void
+gdaui_basic_form_set_unknown_color (GdauiBasicForm *form, gdouble red, gdouble green,
+				    gdouble blue, gdouble alpha)
+{
+	g_return_if_fail (GDAUI_IS_BASIC_FORM (form));
+	form->priv->red = red;
+	form->priv->green = green;
+	form->priv->blue = blue;
+	form->priv->alpha = alpha;
+
+	GSList *list;
+	for (list = form->priv->s_entries; list; list = list->next) {
+		SingleEntry *se;
+		se = (SingleEntry *) list->data;
+		gdaui_data_entry_set_unknown_color (se->entry, form->priv->red,
+						    form->priv->green, form->priv->blue,
+						    form->priv->alpha);
+	}
+}
