@@ -35,6 +35,16 @@ G_BEGIN_DECLS
 typedef struct _GdauiDataEntry        GdauiDataEntry;
 typedef struct _GdauiDataEntryIface   GdauiDataEntryIface;
 
+/* error reporting */
+extern GQuark gdaui_data_entry_error_quark (void);
+#define GDAUI_DATA_ENTRY_ERROR gdaui_data_entry_error_quark ()
+
+typedef enum
+{
+	GDAUI_DATA_ENTRY_FILE_NOT_FOUND_ERROR,
+	GDAUI_DATA_ENTRY_INVALID_DATA_ERROR
+} GdauiDataEntryError;
+
 /* struct for the interface */
 struct _GdauiDataEntryIface
 {
@@ -63,11 +73,13 @@ struct _GdauiDataEntryIface
 	void            (*grab_focus)            (GdauiDataEntry *de);
 
 	/* another signal */
-	void            (* expand_changed)       (GdauiDataEntry *de);
+	void            (*expand_changed)        (GdauiDataEntry *de);
+
+	void            (*set_unknown_color)     (GdauiDataEntry *de, gdouble red, gdouble green,
+						  gdouble blue, gdouble alpha);
 
 	/*< private >*/
 	/* Padding for future expansion */
-        void (*_gdaui_reserved2) (void);
         void (*_gdaui_reserved3) (void);
         void (*_gdaui_reserved4) (void);
 };
@@ -98,6 +110,9 @@ void            gdaui_data_entry_set_editable           (GdauiDataEntry *de, gbo
 gboolean        gdaui_data_entry_get_editable           (GdauiDataEntry *de);
 void            gdaui_data_entry_grab_focus             (GdauiDataEntry *de);
 
+void            gdaui_data_entry_set_unknown_color      (GdauiDataEntry *de, gdouble red, gdouble green,
+						 	 gdouble blue, gdouble alpha);
+
 G_END_DECLS
 
 #endif
