@@ -292,6 +292,9 @@ gdaui_raw_form_init (GdauiRawForm *wid)
 
 	wid->priv->filter = NULL;
 	wid->priv->filter_window = NULL;
+
+	/* raw forms' default unknown color */
+	gdaui_basic_form_set_unknown_color ((GdauiBasicForm*) wid, -1., -1., -1., -1.);
 }
 
 /**
@@ -459,7 +462,7 @@ gdaui_raw_form_set_property (GObject *object,
 					g_object_set (object, "paramlist", form->priv->iter, NULL);
 					gda_data_proxy_set_sample_size (form->priv->proxy, 0);
 
-					/* handle invalie iterators' GdaHolder */
+					/* handle invalid iterators' GdaHolder */
 					GSList *list;
 					for (list = GDA_SET (form->priv->iter)->holders; list; list = list->next) {
 						GtkWidget *entry;
@@ -574,10 +577,14 @@ proxy_changed_cb (G_GNUC_UNUSED GdaDataProxy *proxy, GdauiRawForm *form)
 static void
 proxy_reset_cb (GdaDataProxy *proxy, GdauiRawForm *form)
 {
+	gint row;
+	row = gda_data_model_iter_get_row (form->priv->iter);
 	g_object_ref (G_OBJECT (proxy));
 	g_object_set (G_OBJECT (form), "model", proxy, NULL);
 	g_object_unref (G_OBJECT (proxy));
-	g_signal_emit_by_name (G_OBJECT (form), "selection-changed");
+	if (row >= 0)
+		gda_data_model_iter_move_to_row (form->priv->iter, row);
+	iter_row_changed_cb (form->priv->iter, gda_data_model_iter_get_row (form->priv->iter), form);
 }
 
 static void
@@ -919,6 +926,7 @@ action_filter_cb (G_GNUC_UNUSED GtkAction *action, GdauiRawForm *form)
 
 	/* move the filter window to a correct location */
 	/* FIXME: let the user specify the position function like GtkTreeView -> search_position_func() */
+	gtk_widget_show (form->priv->filter_window);
 	gtk_grab_add (form->priv->filter_window);
 	filter_position_func (GTK_WIDGET (form), form->priv->filter_window, NULL);
 	gtk_widget_show (form->priv->filter_window);
