@@ -69,6 +69,7 @@ typedef struct {
 struct _GdauiRtEditorPriv
 {
 	GtkTextView    *textview;
+	gdouble         vadj_value;
 	GtkTextBuffer  *textbuffer;
 	GtkWidget      *toolbar;
 	GtkActionGroup *actions_group;
@@ -287,6 +288,13 @@ text_view_realized_cb (GtkWidget *tv, GdauiRtEditor *rte)
 	}
 }
 
+static gboolean
+focus_changed_cb (GtkWidget *textview, GdkEventFocus *ev, GdauiRtEditor *rte)
+{
+	show_hide_toolbar (rte);
+	return FALSE;
+}
+
 static void
 gdaui_rt_editor_init (GdauiRtEditor *rte)
 {
@@ -307,6 +315,7 @@ gdaui_rt_editor_init (GdauiRtEditor *rte)
 
 	rte->priv = g_new0 (GdauiRtEditorPriv, 1);
 	rte->priv->sw = sw;
+	rte->priv->vadj_value = 0.;
 	rte->priv->saved_for_help = NULL;
 	rte->priv->enable_changed_signal = TRUE;
 	rte->priv->no_background = FALSE;
@@ -324,6 +333,10 @@ gdaui_rt_editor_init (GdauiRtEditor *rte)
 				G_CALLBACK (insert_text_after_cb), rte);
 	g_signal_connect (rte->priv->textview, "populate-popup",
 			  G_CALLBACK (populate_popup_cb), rte);
+	g_signal_connect (rte->priv->textview, "focus-in-event",
+			  G_CALLBACK (focus_changed_cb), rte);
+	g_signal_connect (rte->priv->textview, "focus-out-event",
+			  G_CALLBACK (focus_changed_cb), rte);
 
 	/* tags. REM: leave the LIST* and BULLET tags defined 1st as they will be with less priority
 	 * and it affects the result returned by gtk_text_iter_get_tags() */
@@ -389,7 +402,8 @@ gdaui_rt_editor_init (GdauiRtEditor *rte)
 	gtk_toolbar_set_icon_size (GTK_TOOLBAR (toolbar), GTK_ICON_SIZE_MENU);
 	rte->priv->toolbar = toolbar;
 	gtk_box_pack_end (GTK_BOX (rte), toolbar, FALSE, FALSE, 0);
-	gtk_widget_show (toolbar);
+
+	show_hide_toolbar (rte);
 }
 
 /**
@@ -1371,8 +1385,15 @@ get_markup_token (GtkTextIter *iter, gint *out_nb_spaces_before, GtkTextIter *ou
 	else if (c == '/') {
 		c = get_char_at_iter (&inti, TRUE);
 		if (c == '/') {
-			SET_OUT;
-			return MARKUP_ITALIC;
+			GtkTextIter previ;
+			previ = inti;
+			if (gtk_text_iter_backward_char (&previ) &&
+			    gtk_text_iter_backward_char (&previ) &&
+			    (get_char_at_iter (&previ, FALSE) == ':')) {}
+			else {
+				SET_OUT;
+				return MARKUP_ITALIC;
+			}
 		}
 	}
 	else if (c == '_') {
@@ -1948,11 +1969,30 @@ show_hide_toolbar (GdauiRtEditor *editor)
 {
 	gboolean enable_markup = TRUE;
 	GtkAction *action;
-	
-	if (gtk_text_view_get_editable (editor->priv->textview))
+	gboolean doshow = FALSE;
+
+	if (gtk_text_view_get_editable (editor->priv->textview) &&
+	    gtk_widget_has_focus (GTK_WIDGET (editor->priv->textview)))
+		doshow = TRUE;
+
+	if (doshow) {
 		gtk_widget_show (editor->priv->toolbar);
-	else
+		if (editor->priv->sw && (editor->priv->vadj_value != 0.)) {
+			GtkAdjustment *adj;	
+			adj = gtk_scrolled_window_get_vadjustment (GTK_SCROLLED_WINDOW (editor->priv->sw));
+			while (gtk_events_pending ())
+				gtk_main_iteration ();
+			gtk_adjustment_set_value (adj, editor->priv->vadj_value);
+		}
+	}
+	else {
+		if (editor->priv->sw) {
+			GtkAdjustment *adj;
+			adj = gtk_scrolled_window_get_vadjustment (GTK_SCROLLED_WINDOW (editor->priv->sw));
+			editor->priv->vadj_value = gtk_adjustment_get_value (adj);
+		}
 		gtk_widget_hide (editor->priv->toolbar);
+	}
 
 	if (editor->priv->show_markup ||
 	    ! gtk_text_view_get_editable (editor->priv->textview))
