@@ -275,12 +275,11 @@ _gdaui_utility_proxy_compute_values_for_group (GdauiSetGroup *group, GdauiDataSt
 					       GdaDataModelIter *model_iter, 
 					       GtkTreeIter *tree_iter, gboolean model_values)
 {
-	gint offset;
 	GList *retval = NULL;
+#ifdef PROXY_STORE_EXTRA_VALUES
 	GdaDataProxy *proxy;
-
 	proxy = gdaui_data_store_get_proxy (store);
-	offset = gda_data_proxy_get_proxied_model_n_cols (proxy);
+#endif
 	if (!model_values) {
 		GSList *list;
 		GValue *value;
@@ -294,13 +293,17 @@ _gdaui_utility_proxy_compute_values_for_group (GdauiSetGroup *group, GdauiDataSt
 		}
 	}
 	else {
-		gint col, i, proxy_row;
+		gint col, i;
+#ifdef PROXY_STORE_EXTRA_VALUES
+		gint proxy_row;
+#endif
 		GdauiSetSource *source;
 		const GValue *value;
 		gboolean slow_way = FALSE;
 		gboolean ret_null = FALSE;
-
+#ifdef PROXY_STORE_EXTRA_VALUES
 		proxy_row = gdaui_data_store_get_row_from_iter (store, tree_iter);
+#endif
 		source = group->source;
 		for (i = 0 ; (i < source->shown_n_cols)  && !ret_null; i++) {
 			col = source->shown_cols_index[i];
@@ -396,7 +399,7 @@ create_data_error_dialog (GdauiDataProxy *form, gboolean with_question, gboolean
 		if (with_question)
 			msg2 =_("You may now choose to correct it, or to discard "
 				"the modifications.\n\n"
-				"What do you wan to do?");
+				"What do you want to do?");
 		else
 			msg2 = _("please correct it and try again, or discard "
 				 "the modifications.");
@@ -422,6 +425,7 @@ create_data_error_dialog (GdauiDataProxy *form, gboolean with_question, gboolean
 		label = gtk_label_new (filled_error->message);
 		gtk_box_pack_start (GTK_BOX (gtk_dialog_get_content_area (GTK_DIALOG (dlg))),
 				    label, TRUE, TRUE, 0);
+		gtk_widget_show (label);
 	}
 
 	return dlg;
