@@ -362,7 +362,6 @@ gda_data_model_reset (GdaDataModel *model)
  * re-enable notifications again, you should call the
  * #gda_data_model_thaw function.
  *
- * Virtual: i_set_notify
  */
 void
 gda_data_model_freeze (GdaDataModel *model)
@@ -2019,7 +2018,6 @@ gda_data_model_import_from_model (GdaDataModel *to, GdaDataModel *from,
 					value = (GValue *) (avlist->data);
 				}
 			}
-			g_assert (value);
 
 			values = g_list_prepend (values, value);
 
