@@ -198,7 +198,7 @@ gda_statement_new (void)
 	return GDA_STATEMENT (obj);
 }
 
-const GdaSqlStatement *
+GdaSqlStatement *
 _gda_statement_get_internal_struct (GdaStatement *stmt)
 {
 	return stmt->priv->internal_struct;
@@ -1729,7 +1729,11 @@ default_render_function (GdaSqlFunction *func, GdaSqlRenderingContext *context,
 			g_string_append (string, ", ");
 		str = context->render_expr (list->data, context, NULL, NULL, error);
 		if (!str) goto err;
+		if (((GdaSqlExpr*) list->data)->select)
+			g_string_append_c (string, '(');
 		g_string_append (string, str);
+		if (((GdaSqlExpr*) list->data)->select)
+			g_string_append_c (string, ')');
 		g_free (str);
 	}
 	g_string_append_c (string, ')');
