@@ -194,7 +194,7 @@ identifier_add_quotes (const gchar *str)
         *retval = '"';
         for (rptr = retval+1, sptr = str; *sptr; sptr++, rptr++) {
                 if (*sptr == '"') {
-                        *rptr = '\\';
+                        *rptr = '"';
                         rptr++;
                         *rptr = *sptr;
                 }
@@ -272,7 +272,17 @@ sqlite_remove_quotes (gchar *str)
 					return str;
 				}
 			}
-			if (*ptr == '\\') {
+			else if (*ptr == '"') {
+				if (*(ptr+1) == '"') {
+					g_memmove (ptr+1, ptr+2, total - offset);
+					offset += 2;
+				}
+				else {
+					*str = 0;
+					return str;
+				}
+			}
+			else if (*ptr == '\\') {
 				if (*(ptr+1) == '\\') {
 					g_memmove (ptr+1, ptr+2, total - offset);
 					offset += 2;
@@ -307,12 +317,8 @@ _gda_sqlite_identifier_quote (G_GNUC_UNUSED GdaServerProvider *provider, GdaConn
         GdaSqlReservedKeywordsFunc kwfunc;
         SqliteConnectionData *cdata = NULL;
 
-        if (cnc) {
+        if (cnc)
                 cdata = (SqliteConnectionData*) gda_connection_internal_get_provider_data (cnc);
-                if (!cdata)
-                        return NULL;
-        }
-
         kwfunc = _gda_sqlite_get_reserved_keyword_func ();
 
 	if (for_meta_store) {
