@@ -31,12 +31,17 @@
 
 struct _GdaVconnectionDataModelPrivate {
 	GSList *table_data_list; /* list of GdaVConnectionTableData structures */
+
+	GMutex       *lock_context;
+	GdaStatement *executed_stmt;
 };
 
 static void gda_vconnection_data_model_class_init (GdaVconnectionDataModelClass *klass);
 static void gda_vconnection_data_model_init       (GdaVconnectionDataModel *cnc, GdaVconnectionDataModelClass *klass);
 static void gda_vconnection_data_model_dispose   (GObject *object);
 
+static gboolean get_rid_of_vtable (GdaVconnectionDataModel *cnc, GdaVConnectionTableData *td, gboolean force, GError **error);
+
 enum {
 	VTABLE_CREATED,
 	VTABLE_DROPPED,
@@ -134,6 +139,7 @@ gda_vconnection_data_model_init (GdaVconnectionDataModel *cnc, G_GNUC_UNUSED Gda
 {
 	cnc->priv = g_new (GdaVconnectionDataModelPrivate, 1);
 	cnc->priv->table_data_list = NULL;
+	cnc->priv->lock_context = g_mutex_new ();
 
 	g_object_set (G_OBJECT (cnc), "cnc-string", "_IS_VIRTUAL=TRUE", NULL);
 }
@@ -147,9 +153,14 @@ gda_vconnection_data_model_dispose (GObject *object)
 
 	/* free memory */
 	if (cnc->priv) {
+		while (cnc->priv->table_data_list) {
+			GdaVConnectionTableData *td;
+			td = (GdaVConnectionTableData *) cnc->priv->table_data_list->data;
+			get_rid_of_vtable (cnc, td, TRUE, NULL);
+		}
 		gda_connection_close_no_warning ((GdaConnection *) cnc);
-		g_assert (!cnc->priv->table_data_list);
 
+		g_mutex_free (cnc->priv->lock_context);
 		g_free (cnc->priv);
 		cnc->priv = NULL;
 	}
@@ -320,7 +331,9 @@ gda_vconnection_data_model_add (GdaVconnectionDataModel *cnc, GdaVconnectionData
 	rc = SQLITE3_CALL (sqlite3_exec) (scnc->connection, str, NULL, 0, &zErrMsg);
 	g_free (str);
 	if (rc != SQLITE_OK) {
-		g_set_error (error, 0, 0, "%s", zErrMsg);
+		g_set_error (error, GDA_SERVER_PROVIDER_ERROR,
+			     GDA_SERVER_PROVIDER_INTERNAL_ERROR,
+			     "%s", zErrMsg);
 		SQLITE3_CALL (sqlite3_free) (zErrMsg);
 		gda_vconnection_data_model_table_data_free (td);
 		cnc->priv->table_data_list = g_slist_remove (cnc->priv->table_data_list, td);
@@ -335,6 +348,44 @@ gda_vconnection_data_model_add (GdaVconnectionDataModel *cnc, GdaVconnectionData
 	return retval;
 }
 
+static gboolean
+get_rid_of_vtable (GdaVconnectionDataModel *cnc, GdaVConnectionTableData *td, gboolean force, GError **error)
+{
+	gchar *str;
+	int rc;
+	char *zErrMsg = NULL;
+	gboolean allok = TRUE;
+
+	SqliteConnectionData *scnc;
+	scnc = (SqliteConnectionData*) gda_connection_internal_get_provider_data ((GdaConnection *) cnc);
+	if (!scnc && !force)
+		return FALSE;
+
+	if (scnc) {
+		str = g_strdup_printf ("DROP TABLE %s", td->table_name);
+		rc = SQLITE3_CALL (sqlite3_exec) (scnc->connection, str, NULL, 0, &zErrMsg);
+		g_free (str);
+
+		if (rc != SQLITE_OK) {
+			g_set_error (error, GDA_SERVER_PROVIDER_ERROR,
+				     GDA_SERVER_PROVIDER_INTERNAL_ERROR,
+				     "%s", zErrMsg);
+			SQLITE3_CALL (sqlite3_free) (zErrMsg);
+			allok = FALSE;
+			if (!force)
+				return FALSE;
+		}
+	}
+
+	/* clean the cnc->priv->table_data_list list */
+	cnc->priv->table_data_list = g_slist_remove (cnc->priv->table_data_list, td);
+	g_signal_emit (G_OBJECT (cnc), gda_vconnection_data_model_signals[VTABLE_DROPPED], 0,
+		       td->table_name);
+	/*g_print ("Virtual connection: removed table %s (%p)\n", td->table_name, td->spec->data_model);*/
+	gda_vconnection_data_model_table_data_free (td);
+
+	return allok;
+}
 
 /**
  * gda_vconnection_data_model_remove:
@@ -352,48 +403,18 @@ gda_vconnection_data_model_remove (GdaVconnectionDataModel *cnc, const gchar *ta
 {
 	GdaVConnectionTableData *td;
 
-	GdaVirtualProvider *prov;
-	gchar *str;
-	int rc;
-	char *zErrMsg = NULL;
-	gboolean retval = TRUE;
-	SqliteConnectionData *scnc;
-
 	g_return_val_if_fail (GDA_IS_VCONNECTION_DATA_MODEL (cnc), FALSE);
 	g_return_val_if_fail (table_name && *table_name, FALSE);
 
-	scnc = (SqliteConnectionData*) gda_connection_internal_get_provider_data ((GdaConnection *) cnc);
-	if (!scnc) 
-		return FALSE;
-
 	td = gda_vconnection_get_table_data_by_name (cnc, table_name);
 	if (!td) {
-		g_set_error (error, 0, 0, "%s",
-			     _("Table to remove not found"));
+		g_set_error (error, GDA_SERVER_PROVIDER_ERROR,
+			     GDA_SERVER_PROVIDER_MISUSE_ERROR,
+			     "%s", _("Table to remove not found"));
 		return FALSE;
 	}
-	
-	prov = (GdaVirtualProvider *) gda_connection_get_provider (GDA_CONNECTION (cnc));
-	str = g_strdup_printf ("DROP TABLE %s", td->table_name);
-	rc = SQLITE3_CALL (sqlite3_exec) (scnc->connection, str, NULL, 0, &zErrMsg);
-	g_free (str);
 
-	if (rc != SQLITE_OK) {
-		g_set_error (error, 0, 0, "%s", zErrMsg);
-		SQLITE3_CALL (sqlite3_free) (zErrMsg);
-		return FALSE;
-	}
-	else {
-		/* clean the cnc->priv->table_data_list list */
-		cnc->priv->table_data_list = g_slist_remove (cnc->priv->table_data_list, td);
-		g_signal_emit (G_OBJECT (cnc), gda_vconnection_data_model_signals[VTABLE_DROPPED], 0,
-			       td->table_name);
-		/*g_print ("Virtual connection: removed table %s (%p)\n", td->table_name, td->spec->data_model);*/
-		gda_vconnection_data_model_table_data_free (td);
-		return TRUE;
-	}
-
-	return retval;
+	return get_rid_of_vtable (cnc, td, FALSE, error);
 }
 
 /**
@@ -491,20 +512,19 @@ void
 gda_vconnection_data_model_foreach (GdaVconnectionDataModel *cnc, 
 				    GdaVconnectionDataModelFunc func, gpointer data)
 {
-	GSList *list, *next;
+	GSList *copy, *list;
 	g_return_if_fail (GDA_IS_VCONNECTION_DATA_MODEL (cnc));
 	g_return_if_fail (cnc->priv);
 
-	if (!func)
+	if (!func || !cnc->priv->table_data_list)
 		return;
 
-	list = cnc->priv->table_data_list;
-	while (list) {
+	copy = g_slist_copy (cnc->priv->table_data_list);
+	for (list = copy; list; list = list->next) {
 		GdaVConnectionTableData *td = (GdaVConnectionTableData*) list->data;
-		next = list->next;
 		func (td->spec->data_model, td->table_name, data);
-		list = next;
 	}
+	g_slist_free (copy);
 }
 
 /* 
@@ -571,5 +591,111 @@ gda_vconnection_data_model_table_data_free (GdaVConnectionTableData *td)
 		if (td->modif_stmt[i])
 			g_object_unref (td->modif_stmt[i]);
 	}
+
+	if (td->context.hash)
+		g_hash_table_destroy (td->context.hash);
 	g_free (td);
 }
+
+static void
+vcontext_object_weak_notify_cb (VContext *context, GObject *old_context_object)
+{
+	g_assert (context);
+	g_mutex_lock (context->vtable->context.mutex);
+	context->context_object = NULL;
+	g_hash_table_remove (context->vtable->context.hash, old_context_object);
+	g_mutex_unlock (context->vtable->context.mutex);
+}
+
+static void
+vcontext_free (VContext *context)
+{
+	if (context->context_object)
+		g_object_weak_unref (context->context_object,
+				     (GWeakNotify) vcontext_object_weak_notify_cb, context);
+	if (context->context_data) {
+		guint i;
+		for (i = 0; i < context->context_data->len; i++) {
+			VirtualFilteredData *data;
+			data = g_array_index (context->context_data, VirtualFilteredData*, i);
+			_gda_vconnection_virtual_filtered_data_unref (data);
+		}
+		g_array_free (context->context_data, TRUE);
+	}
+	g_free (context);
+#ifdef DEBUG_VCONTEXT
+	g_print ("VCFree %p\n", context);
+#endif
+}
+
+void
+_gda_vconnection_set_working_obj (GdaVconnectionDataModel *cnc, GObject *obj)
+{
+	GSList *list;
+	if (obj) {
+		g_mutex_lock (cnc->priv->lock_context);
+		for (list = cnc->priv->table_data_list; list; list = list->next) {
+			GdaVConnectionTableData *td = (GdaVConnectionTableData*) list->data;
+			VContext *vc = NULL;
+			
+			g_assert (!td->context.current_vcontext);
+			td->context.mutex = cnc->priv->lock_context;
+			if (! td->context.hash)
+				td->context.hash = g_hash_table_new_full (g_direct_hash, g_direct_equal,
+									  NULL, (GDestroyNotify) vcontext_free);
+			else
+				vc = g_hash_table_lookup (td->context.hash, obj);
+
+			if (! vc) {
+				vc = g_new0 (VContext, 1);
+				vc->context_object = obj;
+				vc->context_data = g_array_new (FALSE, FALSE,
+								sizeof (VirtualFilteredData*));
+				vc->vtable = td;
+				g_object_weak_ref (obj, (GWeakNotify) vcontext_object_weak_notify_cb, vc);
+				g_hash_table_insert (td->context.hash, obj, vc);
+#ifdef DEBUG_VCONTEXT
+				g_print ("VCNew %p\n", vc);
+#endif
+			}
+			td->context.current_vcontext = vc;
+		}
+	}
+	else {
+		for (list = cnc->priv->table_data_list; list; list = list->next) {
+			GdaVConnectionTableData *td = (GdaVConnectionTableData*) list->data;
+
+			g_assert (td->context.current_vcontext);
+			td->context.current_vcontext = NULL;
+		}
+		g_mutex_unlock (cnc->priv->lock_context);
+	}
+}
+
+void
+_gda_vconnection_change_working_obj (GdaVconnectionDataModel *cnc, GObject *obj)
+{
+	GSList *list;
+
+	for (list = cnc->priv->table_data_list; list; list = list->next) {
+		GdaVConnectionTableData *td = (GdaVConnectionTableData*) list->data;
+		if (!td->context.hash)
+			continue;
+
+		g_assert (td->context.current_vcontext);
+
+		VContext *ovc, *nvc;
+		ovc = td->context.current_vcontext;
+		nvc = g_new0 (VContext, 1);
+		nvc->context_object = obj;
+		nvc->vtable = ovc->vtable;
+		nvc->context_data = ovc->context_data;
+		ovc->context_data = NULL;
+		g_object_weak_ref (obj, (GWeakNotify) vcontext_object_weak_notify_cb, nvc);
+		g_hash_table_insert (td->context.hash, obj, nvc);
+#ifdef DEBUG_VCONTEXT
+		g_print ("VCNew %p\n", nvc);
+#endif
+		g_hash_table_remove (td->context.hash, ovc->context_object);
+	}
+}
