@@ -1,23 +1,20 @@
 /*
- * Copyright (C) 2011 The GNOME Foundation
+ * Copyright (C) 2011 Vivien Malerba <malerba@gnome-db.org>
  *
- * AUTHORS:
- *      Vivien Malerba <malerba@gnome-db.org>
+ * This library is free software; you can redistribute it and/or
+ * modify it under the terms of the GNU Lesser General Public
+ * License as published by the Free Software Foundation; either
+ * version 2 of the License, or (at your option) any later version.
  *
- * This Library is free software; you can redistribute it and/or
- * modify it under the terms of the GNU Library General Public License as
- * published by the Free Software Foundation; either version 2 of the
- * License, or (at your option) any later version.
- *
- * This Library is distributed in the hope that it will be useful,
+ * This library is distributed in the hope that it will be useful,
  * but WITHOUT ANY WARRANTY; without even the implied warranty of
  * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
- * Library General Public License for more details.
+ * Lesser General Public License for more details.
  *
- * You should have received a copy of the GNU Library General Public
- * License along with this Library; see the file COPYING.LIB.  If not,
- * write to the Free Software Foundation, Inc., 59 Temple Place - Suite 330,
- * Boston, MA 02111-1307, USA.
+ * You should have received a copy of the GNU Lesser General Public
+ * License along with this library; if not, write to the
+ * Free Software Foundation, Inc., 51 Franklin St, Fifth Floor,
+ * Boston, MA  02110-1301, USA.
  */
 
 #include <string.h>
@@ -95,6 +92,7 @@ static void row_multiplier_set_holders (RowMultiplier *rm);
 struct _GdaDataModelLdapPrivate {
 	GdaConnection      *cnc;
 	gchar              *base_dn;
+	gboolean            use_rdn;
 	gchar              *filter;
 	GArray             *attributes;
 	GdaLdapSearchScope  scope;
@@ -121,7 +119,8 @@ enum {
 	PROP_BASE,
 	PROP_FILTER,
 	PROP_ATTRIBUTES,
-	PROP_SCOPE
+	PROP_SCOPE,
+	PROP_USE_RDN
 };
 
 static void gda_data_model_ldap_class_init (GdaDataModelLdapClass *klass);
@@ -198,6 +197,8 @@ gda_data_model_ldap_init (GdaDataModelLdap *model,
 
 	model->priv = g_new0 (GdaDataModelLdapPrivate, 1);
 	model->priv->cnc = NULL;
+	model->priv->base_dn = NULL;
+	model->priv->use_rdn = FALSE;
 	model->priv->filter = g_strdup ("(objectClass=*)");
 	model->priv->iter_row = -1;
 	model->priv->default_mv_action = MULTIPLE_VALUE_ACTION_SET_INVALID;
@@ -256,6 +257,11 @@ gda_data_model_ldap_class_init (GdaDataModelLdapClass *klass)
 							   G_PARAM_WRITABLE | G_PARAM_READABLE |
 							   G_PARAM_CONSTRUCT_ONLY));
 
+	g_object_class_install_property (object_class, PROP_USE_RDN,
+                                         g_param_spec_boolean ("use-rdn", NULL, "Return Relative DN instead of complete DN",
+							       FALSE, G_PARAM_WRITABLE | G_PARAM_READABLE));
+
+
 	/* virtual functions */
 	object_class->dispose = gda_data_model_ldap_dispose;
 }
@@ -401,6 +407,12 @@ gda_data_model_ldap_set_property (GObject *object,
 									      &model->priv->attributes,
 									      model->priv->default_mv_action,
 									      &model->priv->column_mv_actions);
+				if (model->priv->use_rdn)
+					gda_column_set_description (GDA_COLUMN (model->priv->columns->data),
+								    _("Relative distinguished name"));
+				else
+					gda_column_set_description (GDA_COLUMN (model->priv->columns->data),
+								    _("Distinguished name"));
 				model->priv->n_columns = g_list_length (model->priv->columns);
 			}
 			break;
@@ -408,6 +420,15 @@ gda_data_model_ldap_set_property (GObject *object,
 		case PROP_SCOPE:
 			model->priv->scope = g_value_get_int (value);
 			break;
+		case PROP_USE_RDN:
+			model->priv->use_rdn = g_value_get_boolean (value);
+			if (model->priv->columns && model->priv->use_rdn)
+				gda_column_set_description (GDA_COLUMN (model->priv->columns->data),
+							    _("Relative distinguished name"));
+			else
+				gda_column_set_description (GDA_COLUMN (model->priv->columns->data),
+							    _("Distinguished name"));
+			break;
 		default:
 			G_OBJECT_WARN_INVALID_PROPERTY_ID (object, param_id, pspec);
 			break;
@@ -438,6 +459,9 @@ gda_data_model_ldap_get_property (GObject *object,
 		case PROP_SCOPE:
 			g_value_set_int (value, model->priv->scope);
 			break;
+		case PROP_USE_RDN:
+		        g_value_set_boolean (value, model->priv->use_rdn);
+			break;
 		default:
 			G_OBJECT_WARN_INVALID_PROPERTY_ID (object, param_id, pspec);
 			break;
@@ -490,7 +514,6 @@ _ldap_compute_columns (GdaConnection *cnc, const gchar *attributes,
 	gda_column_set_name (col, "dn");
 	gda_column_set_g_type (col, G_TYPE_STRING);
 	gda_column_set_allow_null (col, FALSE);
-	gda_column_set_description (col, _("Distinguished name"));
 	columns = g_list_prepend (NULL, col);
 	g_hash_table_insert (colnames, g_strdup ("dn"), (gpointer) 0x01);
 
@@ -531,6 +554,7 @@ _ldap_compute_columns (GdaConnection *cnc, const gchar *attributes,
 		}
 		col = gda_column_new ();
 		gda_column_set_name (col, sub [0]);
+		gda_column_set_description (col, sub [0]);
 		g_hash_table_insert (colnames, g_strdup (sub [0]), (gpointer) 0x01);
 		gda_column_set_g_type (col, coltype);
 		gda_column_set_allow_null (col, TRUE);
@@ -686,6 +710,18 @@ update_iter_from_ldap_row (GdaDataModelLdap *imodel, GdaDataModelIter *iter)
 	if (attr) {
 		gchar *userdn;
 		if (gda_ldap_parse_dn (attr, &userdn)) {
+			if (imodel->priv->base_dn && *imodel->priv->base_dn &&
+			    imodel->priv->use_rdn &&
+			    g_str_has_suffix (userdn, imodel->priv->base_dn)) {
+				gint i;
+				i = strlen (userdn) - strlen (imodel->priv->base_dn);
+				if (i > 0) {
+					userdn [i] = 0;
+					for (i--; (i >= 0) && (userdn [i] != ','); i--);
+					if ((i >= 0) && userdn [i] == ',')
+						userdn [i] = 0;
+				}
+			}
 			gda_holder_set_value_str (holder, NULL, userdn, NULL);
 			g_free (userdn);
 		}
@@ -853,7 +889,7 @@ update_iter_from_ldap_row (GdaDataModelLdap *imodel, GdaDataModelIter *iter)
 	if (gda_data_model_iter_is_valid (iter)) {
 		imodel->priv->iter_row ++;
 		if ((imodel->priv->iter_row == imodel->priv->n_rows - 1) && imodel->priv->truncated) {
-			GError *e;
+			GError *e = NULL;
 			g_set_error (&e, GDA_DATA_MODEL_ERROR,
 				     GDA_DATA_MODEL_TRUNCATED_ERROR,
 				     _("Truncated result because LDAP server limit encountered"));
@@ -1048,12 +1084,14 @@ gda_data_model_ldap_iter_next (GdaDataModel *model, GdaDataModelIter *iter)
 
 	if (! imodel->priv->cnc) {
 		/* error */
+		gda_data_model_iter_invalidate_contents (iter);
 		return FALSE;
 	}
 
 	cdata = (LdapConnectionData*) gda_virtual_connection_internal_get_provider_data (GDA_VIRTUAL_CONNECTION (imodel->priv->cnc));
 	if (!cdata) {
 		/* error */
+		gda_data_model_iter_invalidate_contents (iter);
 		return FALSE;
 	}
 
@@ -1074,6 +1112,7 @@ gda_data_model_ldap_iter_next (GdaDataModel *model, GdaDataModelIter *iter)
 		cpart = imodel->priv->current_exec;
 		if (! cpart->ldap_msg) {
 			/* error somewhere */
+			gda_data_model_iter_invalidate_contents (iter);
 			return FALSE;
 		}
 
@@ -1104,15 +1143,16 @@ gda_data_model_ldap_iter_next (GdaDataModel *model, GdaDataModelIter *iter)
 
 	if (!imodel->priv->current_exec) {
 		/* execution is over */
-		g_signal_emit_by_name (iter, "end-of-data");
+		gda_data_model_iter_invalidate_contents (iter);
                 g_object_set (G_OBJECT (iter), "current-row", -1, NULL);
 		if (imodel->priv->truncated) {
-			GError *e;
+			GError *e = NULL;
 			g_set_error (&e, GDA_DATA_MODEL_ERROR,
 				     GDA_DATA_MODEL_TRUNCATED_ERROR,
 				     _("Truncated result because LDAP server limit encountered"));
 			add_exception (imodel, e);
 		}
+		g_signal_emit_by_name (iter, "end-of-data");
                 return FALSE;
 	}
 
