@@ -2417,7 +2417,7 @@ identifier_add_quotes (const gchar *str)
         *retval = '"';
         for (rptr = retval+1, sptr = str; *sptr; sptr++, rptr++) {
                 if (*sptr == '"') {
-                        *rptr = '\\';
+                        *rptr = '"';
                         rptr++;
                         *rptr = *sptr;
                 }
@@ -2468,7 +2468,6 @@ pg_remove_quotes (gchar *str)
 	if ((delim != '\'') && (delim != '"'))
 		return str;
 
-
         total = strlen (str);
         if (str[total-1] == delim) {
 		/* string is correctly terminated */
@@ -2495,7 +2494,17 @@ pg_remove_quotes (gchar *str)
                                 return str;
                         }
                 }
-                if (*ptr == '\\') {
+                else if (*ptr == '"') {
+                        if (*(ptr+1) == '"') {
+                                g_memmove (ptr+1, ptr+2, total - offset);
+                                offset += 2;
+                        }
+                        else {
+				*str = 0;
+				return str;
+                        }
+                }
+		else if (*ptr == '\\') {
                         if (*(ptr+1) == '\\') {
                                 g_memmove (ptr+1, ptr+2, total - offset);
                                 offset += 2;
@@ -2529,11 +2538,8 @@ gda_postgresql_identifier_quote (G_GNUC_UNUSED GdaServerProvider *provider, GdaC
         GdaSqlReservedKeywordsFunc kwfunc;
         PostgresConnectionData *cdata = NULL;
 
-        if (cnc) {
+        if (cnc)
                 cdata = (PostgresConnectionData*) gda_connection_internal_get_provider_data (cnc);
-                if (!cdata)
-                        return NULL;
-        }
 
         kwfunc = _gda_postgres_reuseable_get_reserved_keywords_func
 		(cdata ? (GdaProviderReuseable*) cdata->reuseable : NULL);
