@@ -437,13 +437,10 @@ gda_postgres_recordset_fetch_random (GdaDataSelect *model, GdaRow **prow, gint r
 {
 	GdaPostgresRecordset *imodel = (GdaPostgresRecordset *) model;
 
-	if (*prow)
-		return TRUE;
-
 	if (!imodel->priv->pg_res) {
 		g_set_error (error, GDA_SERVER_PROVIDER_ERROR, GDA_SERVER_PROVIDER_INTERNAL_ERROR,
 			     "%s", _("Internal error"));
-		return FALSE;
+		return TRUE;
 	}
 
 	*prow = new_row_from_pg_res (imodel, rownum, error);
@@ -486,23 +483,19 @@ gda_postgres_recordset_store_all (GdaDataSelect *model, GError **error)
  * Create a new filled #GdaRow object for the next cursor row, and put it into *prow.
  *
  * Each new #GdaRow created is referenced only by imodel->priv->tmp_row (the #GdaDataSelect implementation
- * never keeps a reference to it). Before a new #GdaRow gets created, the previous one, if set, is discarded.
+ * never keeps a reference to it).
  */
 static gboolean
 gda_postgres_recordset_fetch_next (GdaDataSelect *model, GdaRow **prow, gint rownum, GError **error)
 {
 	GdaPostgresRecordset *imodel = (GdaPostgresRecordset*) model;
 
-	if (*prow)
-		return TRUE;
-
 	if (row_is_in_current_pg_res (imodel, rownum)) {
 		if (imodel->priv->tmp_row)
 			set_prow_with_pg_res (imodel, imodel->priv->tmp_row, rownum - imodel->priv->pg_res_inf, error);
 		else
 			imodel->priv->tmp_row = new_row_from_pg_res (imodel, rownum - imodel->priv->pg_res_inf, error);
 		*prow = imodel->priv->tmp_row;
-		return TRUE;
 	}
 	else {
 		gboolean fetch_error = FALSE;
@@ -512,34 +505,28 @@ gda_postgres_recordset_fetch_next (GdaDataSelect *model, GdaRow **prow, gint row
 			else
 				imodel->priv->tmp_row = new_row_from_pg_res (imodel, rownum - imodel->priv->pg_res_inf, error);
 			*prow = imodel->priv->tmp_row;
-			return TRUE;
 		}
-		else
-			return !fetch_error;
 	}
+	return TRUE;
 }
 
 /*
  * Create a new filled #GdaRow object for the previous cursor row, and put it into *prow.
  *
  * Each new #GdaRow created is referenced only by imodel->priv->tmp_row (the #GdaDataSelect implementation
- * never keeps a reference to it). Before a new #GdaRow gets created, the previous one, if set, is discarded.
+ * never keeps a reference to it).
  */
 static gboolean
 gda_postgres_recordset_fetch_prev (GdaDataSelect *model, GdaRow **prow, gint rownum, GError **error)
 {
 	GdaPostgresRecordset *imodel = (GdaPostgresRecordset*) model;
 
-	if (*prow)
-		return TRUE;
-
 	if (row_is_in_current_pg_res (imodel, rownum)) {
 		if (imodel->priv->tmp_row)
 			set_prow_with_pg_res (imodel, imodel->priv->tmp_row, rownum - imodel->priv->pg_res_inf, error);
 		else
 			imodel->priv->tmp_row = new_row_from_pg_res (imodel, rownum - imodel->priv->pg_res_inf, error);
 		*prow = imodel->priv->tmp_row;
-		return TRUE;
 	}
 	else {
 		gboolean fetch_error = FALSE;
@@ -549,27 +536,22 @@ gda_postgres_recordset_fetch_prev (GdaDataSelect *model, GdaRow **prow, gint row
 			else
 				imodel->priv->tmp_row = new_row_from_pg_res (imodel, rownum - imodel->priv->pg_res_inf, error);
 			*prow = imodel->priv->tmp_row;
-			return TRUE;
 		}
-		else
-			return !fetch_error;
 	}
+	return TRUE;
 }
 
 /*
  * Create a new filled #GdaRow object for the cursor row at position @rownum, and put it into *prow.
  *
  * Each new #GdaRow created is referenced only by imodel->priv->tmp_row (the #GdaDataSelect implementation
- * never keeps a reference to it). Before a new #GdaRow gets created, the previous one, if set, is discarded.
+ * never keeps a reference to it).
  */
 static gboolean
 gda_postgres_recordset_fetch_at (GdaDataSelect *model, GdaRow **prow, gint rownum, GError **error)
 {
 	GdaPostgresRecordset *imodel = (GdaPostgresRecordset*) model;
 
-	if (*prow)
-		return TRUE;
-
 	if (imodel->priv->tmp_row) {
 		g_object_unref (imodel->priv->tmp_row);
 		imodel->priv->tmp_row = NULL;
@@ -578,18 +560,15 @@ gda_postgres_recordset_fetch_at (GdaDataSelect *model, GdaRow **prow, gint rownu
 	if (row_is_in_current_pg_res (imodel, rownum)) {
 		*prow = new_row_from_pg_res (imodel, rownum - imodel->priv->pg_res_inf, error);
 		imodel->priv->tmp_row = *prow;
-		return TRUE;
 	}
 	else {
 		gboolean fetch_error = FALSE;
 		if (fetch_row_number_chunk (imodel, rownum, &fetch_error, error)) {
 			*prow = new_row_from_pg_res (imodel, rownum - imodel->priv->pg_res_inf, error);
 			imodel->priv->tmp_row = *prow;
-			return TRUE;
 		}
-		else
-			return !fetch_error;
 	}
+	return TRUE;
 }
 
 
@@ -697,14 +676,73 @@ set_value (GdaConnection *cnc, GdaRow *row, GValue *value, GType type, const gch
 		 */
 		guchar *unescaped;
                 size_t pqlength = 0;
+		PostgresConnectionData *cdata;
+		gboolean valueset = FALSE;
 
-		unescaped = PQunescapeBytea ((guchar*)thevalue, &pqlength);
-		if (unescaped != NULL) {
-			GdaBinary bin;
-			bin.data = unescaped;
-			bin.binary_length = pqlength;
-			gda_value_set_binary (value, &bin);
-			PQfreemem (unescaped);
+		cdata = (PostgresConnectionData*) gda_connection_internal_get_provider_data (cnc);
+		if (cdata) {
+			if ((thevalue[0] == '\\') && (thevalue[1] == 'x')) {
+				guint len;
+				len = strlen (thevalue + 2);
+				if (!(len % 2)) {
+					guint i;
+					const gchar *ptr;
+					pqlength = len / 2;
+					unescaped = g_new (guchar, pqlength);
+					for (i = 0, ptr = thevalue + 2; *ptr; i++, ptr += 2) {
+						gchar c;
+						c = ptr[0];
+						if ((c >= 'a') && (c <= 'z'))
+							unescaped [i] = c - 'a' + 10;
+						else if ((c >= 'A') && (c <= 'Z'))
+							unescaped [i] = c - 'A' + 10;
+						else if ((c >= '0') && (c <= '9'))
+							unescaped [i] = c - '0';
+						else
+							break;
+						unescaped [i] <<= 4;
+
+						c = ptr[1];
+						if ((c >= 'a') && (c <= 'z'))
+							unescaped [i] += c - 'a' + 10;
+						else if ((c >= 'A') && (c <= 'Z'))
+							unescaped [i] += c - 'A' + 10;
+						else if ((c >= '0') && (c <= '9'))
+							unescaped [i] += c - '0';
+						else
+							break;
+					}
+					if (! *ptr) {
+						GdaBinary *bin;
+						bin = g_new (GdaBinary, 1);
+						bin->data = unescaped;
+						bin->binary_length = pqlength;
+						gda_value_take_binary (value, bin);
+						valueset = TRUE;
+					}
+				}
+			}
+			else {
+				unescaped = PQunescapeBytea ((guchar*) thevalue, &pqlength);
+				if (unescaped) {
+					GdaBinary bin;
+					bin.data = unescaped;
+					bin.binary_length = pqlength;
+					gda_value_set_binary (value, &bin);
+					PQfreemem (unescaped);
+					valueset = TRUE;
+				}
+			}
+		}
+		if (!valueset) {
+			gchar *tmp;
+			tmp = g_strndup (thevalue, 20);
+			gda_row_invalidate_value (row, value);
+			g_set_error (error, GDA_SERVER_PROVIDER_ERROR,
+				     GDA_SERVER_PROVIDER_DATA_ERROR,
+				     _("Invalid binary string representation '%s ...'"), 
+				     tmp);
+			g_free (tmp);
 		}
 	}
 	else if (type == GDA_TYPE_BLOB) {
