@@ -1,20 +1,20 @@
 /*
- * Copyright (C) 2009 - 2011 Vivien Malerba
+ * Copyright (C) 2009 - 2011 Vivien Malerba <malerba@gnome-db.org>
+ * Copyright (C) 2010 David King <davidk@openismus.com>
  *
- * This Library is free software; you can redistribute it and/or
- * modify it under the terms of the GNU Library General Public License as
- * published by the Free Software Foundation; either version 2 of the
- * License, or (at your option) any later version.
+ * This program is free software; you can redistribute it and/or
+ * modify it under the terms of the GNU General Public License
+ * as published by the Free Software Foundation; either version 2
+ * of the License, or (at your option) any later version.
  *
- * This Library is distributed in the hope that it will be useful,
+ * This program is distributed in the hope that it will be useful,
  * but WITHOUT ANY WARRANTY; without even the implied warranty of
- * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
- * Library General Public License for more details.
+ * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
+ * GNU General Public License for more details.
  *
- * You should have received a copy of the GNU Library General Public
- * License along with this Library; see the file COPYING.LIB.  If not,
- * write to the Free Software Foundation, Inc., 59 Temple Place - Suite 330,
- * Boston, MA 02111-1307, USA.
+ * You should have received a copy of the GNU General Public License
+ * along with this program; if not, write to the Free Software
+ * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.
  */
 
 #include <string.h>
@@ -36,6 +36,8 @@ static void query_exec_perspective_init (QueryExecPerspective *stmt);
 static void query_exec_perspective_dispose (GObject *object);
 
 static void query_exec_perspective_grab_focus (GtkWidget *widget);
+static void query_exec_add_cb (G_GNUC_UNUSED GtkAction *action, BrowserPerspective *bpers);
+
 
 /* BrowserPerspective interface */
 static void                 query_exec_perspective_perspective_init (BrowserPerspectiveIface *iface);
@@ -213,8 +215,12 @@ fav_selection_changed_cb (G_GNUC_UNUSED GtkWidget *widget, gint fav_id,
 
 	nb = GTK_NOTEBOOK (perspective->priv->notebook);
 	page = gtk_notebook_get_nth_page (nb, gtk_notebook_get_current_page (nb));
-	if (!page)
-		return;
+	if (!page) {
+		query_exec_add_cb (NULL, BROWSER_PERSPECTIVE (perspective));
+		page = gtk_notebook_get_nth_page (nb, gtk_notebook_get_current_page (nb));
+		if (!page)
+			return;
+	}
 	if (IS_QUERY_CONSOLE_PAGE_PAGE (page)) {
 		query_console_page_set_text (QUERY_CONSOLE_PAGE (page), selection, fav_id);
 		gtk_widget_grab_focus (page);
